#
# DivPages.pl
# 2007/09/01 1.10 For Movable Type 4
# 2009/01/07 1.11 Add function
# 2010/02/03 1.20 For Movable Type 5
# 2010/03/04 1.21 Support multi-purpose pagination
# 2017/09/04 1.22 Fix for rebuild queue
# 2018/07/11 2.00 For Movabel Type 7
#
# Copyright(c) by H.Fujimoto
#
package MT::Plugin::DivPages;
use base 'MT::Plugin';

use strict;

use MT;
use MT::Template::Context;
use MT::Plugin;
use MT::Entry;
use MT::ContentStatus;
use MT::Template::Tags::ContentType;
use MT::Util qw ( encode_url encode_html );
our $VERSION = '2.00b2';

# show plugin information to main menu
my $plugin = MT::Plugin::DivPages->new({
    name => 'DivPages',
    version => $VERSION,
    author_name => '<__trans phrase="Hajime Fujimoto">',
    author_link => 'http://www.h-fj.com/blog/',
    description => '<__trans phrase="Divide main page / archive page to multi pages.">',
    l10n_class => 'DivPages::L10N',
});
MT->add_plugin($plugin);

# init_registry
sub init_registry
{
    my $plugin = shift;
    $plugin->registry({
        tags => {
            block => {
                EntriesDivPages => \&entries_div_pages,
                ContentsDivPages => \&entries_div_pages,
                DivPagesLoop => \&div_pages_loop,

                'DivPagesIfMulti?' => \&div_pages_if_multi,

                'DivPagesIfFirstPage?' => \&div_pages_if_first_page,
                'DivPagesIfLastPage?' => \&div_pages_if_last_page,
                'DivPagesIfPrevPage?' => \&div_pages_if_prev_page,
                'DivPagesIfNextPage?' => \&div_pages_if_next_page,

                'DivPagesIfDelimiter?' => \&div_pages_if_delimiter,
                'DivPagesIfCurPage?' => \&div_pages_if_cur_page,

                'DivPagesLoopHeader?' => \&div_pages_loop_header,
                'DivPagesLoopFooter?' => \&div_pages_loop_footer,
                'DivPagesLoopIfPrevPage?' => \&div_pages_loop_if_prev_page,
                'DivPagesLoopIfNextPage?' => \&div_pages_loop_if_next_page,
                'DivPagesLoopIfPrevFirst?' => \&div_pages_loop_if_prev_first,
                'DivPagesLoopIfNextLast?' => \&div_pages_loop_if_next_last,
            },
            function => {
                DivPagesInit => \&div_pages_init,
                DivPagesContentsInit => \&div_pages_init,
                DivPagesCount => \&div_pages_count,
                DivPagesEntryCount => \&div_pages_entry_count,
                DivPagesContentCount => \&div_pages_content_count,
                DivPagesObjectCount => \&div_pages_entry_count,

                DivPagesCurPageLink => \&div_pages_cur_page_link,
                DivPagesCurPageNumber => \&div_pages_cur_page_number,
                DivPagesCurPageFirstEntryNumber => \&div_pages_cur_page_first_entry_number,
                DivPagesCurPageLastEntryNumber => \&div_pages_cur_page_last_entry_number,
                DivPagesCurPageFirstContentNumber => \&div_pages_cur_page_first_entry_number,
                DivPagesCurPageLastContentNumber => \&div_pages_cur_page_last_entry_number,
                DivPagesCurPageFirstObjectNumber => \&div_pages_cur_page_first_entry_number,
                DivPagesCurPageLastObjectNumber => \&div_pages_cur_page_last_entry_number,

                DivPagesNumber => \&div_pages_number,
                DivPagesNumberLink => \&div_pages_number,
                DivPagesLink => \&div_pages_link,
                DivPagesFirstPageLink => \&div_pages_first_page_link,
                DivPagesLastPageLink => \&div_pages_last_page_link,
                DivPagesPrevPageLink => \&div_pages_prev_page_link,
                DivPagesNextPageLink => \&div_pages_next_page_link,
                DivPagesPrevPageNumber => \&div_pages_prev_page_number,
                DivPagesNextPageNumber => \&div_pages_next_page_number,
                DivPagesLastPageNumber => \&div_pages_last_page_number,
                DivPagesBuildParams => \&div_pages_build_params,
                DivPagesCGILink => \&div_pages_cgi_link,
                DivPagesPHPLink => \&div_pages_cgi_link,
            },
        },
    });
}

# MTDivPagesInit tag
sub div_pages_init {
    my ($ctx, $args) = @_;
    my $app = MT->instance;

    # get "per_page" attribute
    my $per_page = $args->{per_page} ||
                   $app->param('per_page') || 10;
    $ctx->stash('per_page', $per_page);
    $ctx->var('dp_per_page', $per_page);

    # get entry count
    my $total_count;
    if (defined($args->{total_count}) || 
        ref $app ne 'MT' && defined($app->param('total_count'))) {
        $total_count = $args->{total_count} || $app->param('total_count')
    }
    elsif (lc $ctx->stash('tag') eq 'divpagesinit') {
        if ($ctx->stash('entries')) {
            $total_count = scalar @{$ctx->stash('entries')};
            if ($args->{sort_order}) {
                my $sort_order = $args->{sort_order} || 'descend';
                my $sort_by = $args->{sort_by} || 'authored_on';
                my @entries = @{$ctx->stash('entries')};
                if ($sort_order eq 'ascend') {
                    @entries = sort { $a->$sort_by() cmp $b->$sort_by() } @entries;
                }
                elsif ($sort_order eq 'descend') {
                    @entries = sort { $b->$sort_by() cmp $a->$sort_by() } @entries;
                }
                $ctx->stash('entries', \@entries);
            }
        }
        else {
            my (%terms, %args);
            $ctx->set_blog_load_context($args, \%terms, \%args)
                or return $ctx->error($ctx->errstr);
            $terms{status} = MT::Entry::RELEASE();
            $total_count = MT::Entry->count(\%terms, \%args);
        }
        $ctx->stash('entry_count', $total_count);
    }
    elsif (lc $ctx->stash('tag') eq 'divpagescontentsinit') {
        if ($ctx->stash('contents')) {
            $total_count = scalar @{$ctx->stash('contents')};
            if ($args->{sort_order}) {
                my $sort_order = $args->{sort_order} || 'descend';
                my $sort_by = $args->{sort_by} || 'authored_on';
                my @contents = @{$ctx->stash('contents')};
                if ($sort_order eq 'ascend') {
                    @contents = sort { $a->$sort_by() cmp $b->$sort_by() } @contents;
                }
                elsif ($sort_order eq 'descend') {
                    @contents = sort { $b->$sort_by() cmp $a->$sort_by() } @contents;
                }
                $ctx->stash('contents', \@contents);
            }
        }
        else {
            my (%blog_terms, %blog_args, %terms, %args);
            $ctx->set_blog_load_context($args, \%blog_terms, \%blog_args)
                or return $ctx->error($ctx->errstr);
            %terms = %blog_terms;
            %args  = %blog_args;
            my $content_type = MT::Template::Tags::ContentType::_get_content_type( $ctx, $args, \%blog_terms );
            my $content_type_id
                = scalar(@$content_type) == 1
                ? $content_type->[0]->id
                : [ map { $_->id } @$content_type ];
            $terms{content_type_id} = $content_type_id;
            $terms{status} = MT::ContentStatus::RELEASE();
            $total_count = MT->model('content_data')->count(\%terms, \%args);
        }
        $ctx->stash('content_count', $total_count);
    }

    # set count of pages
    my $ec_tmp = $total_count - 1;
    my $pages_count = ($ec_tmp - ($ec_tmp % $per_page)) / $per_page + 1;
    $ctx->stash('pages_count', $pages_count);

    # get "?page=x" parameter
    my $cur_page_no;
    if (ref $app ne 'MT') {
        $cur_page_no = $app->param('page') || 1;
    }
    else {
        $cur_page_no = 1;
    }
#    if (defined($app->{query})) {
#        $cur_page_no = $app->{query}->param('page');
#    }
#    else {
#        $cur_page_no = 1;
#    }
    if ($cur_page_no eq 'first') {
        $cur_page_no = 1;
    }
    elsif ($cur_page_no eq 'last') {
        $cur_page_no = $pages_count;
    }
    elsif ($cur_page_no < 1) {
        $cur_page_no = 1;
    }
    elsif ($cur_page_no > $pages_count) {
        $cur_page_no = $pages_count;
    }
    $cur_page_no = int($cur_page_no);
    $ctx->stash('cur_page_no', $cur_page_no);
    $ctx->var('dp_offset', ($cur_page_no - 1) * $per_page);
    $ctx->var('dp_total_count', $total_count);
    '';
}

# MTEntriesDivPages tag
sub entries_div_pages {
    my ($ctx, $args, $cond) = @_;

    my $cur_page_no  = $ctx->stash('cur_page_no');
    my $per_page = $ctx->stash('per_page');
    my $res = '';

    # get attributes
    my $lastn = $args->{'lastn'} || $per_page;
    my $offset = $args->{'offset'} || 0;
    $offset += ($cur_page_no - 1) * $per_page;

    # load entries
#    my $offset = ($cur_page_no - 1) * $per_page;

    # out entries
    if (lc $ctx->stash('tag') eq 'entriesdivpages') {
        $args->{'offset'} = $offset;
        $args->{'lastn'} = $lastn;
        $res = $ctx->invoke_handler('entries', $args, $cond);
    }
    elsif (lc $ctx->stash('tag') eq 'contentsdivpages') {
        $args->{'offset'} = $offset;
        $args->{'limit'} = $lastn;
        $res = $ctx->invoke_handler('contents', $args, $cond);
    }
    $res;
}

# MTDivPagesLoop tag
sub div_pages_loop {
    my ($ctx, $args) = @_;

    my $tokens  = $ctx->stash('tokens');
    my $builder = $ctx->stash('builder');
    my $res = '';
    my @res;

    my ($pages_count, $cur_page_no, $delimiter, $count, $pre_count, $after_count);
    my ($ctr, $page_no, $all_flag, $first_pages, $mid_pages, $last_pages);
    my @page_numbers;

    $pages_count = $ctx->stash('pages_count');
    $all_flag    = $args->{'all'};
    $first_pages = $args->{'first'} || 0;
    $mid_pages   = $args->{'mid'} || 0;
    $last_pages  = $args->{'last'} || 0;
    $delimiter   = $args->{'delimiter'} || '';
    $count       = $args->{'count'} || 0;
    $cur_page_no = $ctx->stash('cur_page_no');
    $pre_count = int($count / 2);
    $after_count = $count - $pre_count - 1;
    $ctx->stash('number_delimiter', $delimiter);
    $all_flag = 1 if (!$first_pages && !$mid_pages && !$last_pages);

    # set page numbers
    if ($count) {
        if ($count > $pages_count) {
            push @page_numbers, 1 ... $pages_count;
        }
        else {
            if ($cur_page_no - $pre_count < 1) {
                push @page_numbers, 1 ... $count;
            }
            elsif ($cur_page_no + $after_count > $pages_count) {
                push @page_numbers, $pages_count - $count + 1 ... $pages_count;
            }
            else {
                push @page_numbers, $cur_page_no - $pre_count ... $cur_page_no + $after_count;
            }
        }
    }
    else {
        if (!$all_flag) {
            push @page_numbers, 1 ... $first_pages if ($first_pages != 0);
            push @page_numbers, $cur_page_no - $mid_pages ... $cur_page_no + $mid_pages if ($mid_pages != 0);
            push @page_numbers, $pages_count - $last_pages + 1 ... $pages_count if ($last_pages != 0);
            my %temp_count;
            @page_numbers = grep(!$temp_count{$_}++, @page_numbers);
            @page_numbers = grep($_ <= $pages_count, @page_numbers);
            @page_numbers = grep($_ > 0, @page_numbers);
            @page_numbers = sort { $a <=> $b } @page_numbers;
        }
        else {
            push @page_numbers, 1 ... $pages_count;
        }
    }

    # out page numbers
    $ctr = 0;
    my $page_numbers_count = scalar(@page_numbers);
    local $ctx->{__stash}{div_pages_loop_if_prev_page} = ($page_numbers[0] != 1);
    local $ctx->{__stash}{div_pages_loop_if_next_page} = ($page_numbers[$#page_numbers] != $pages_count);
    local $ctx->{__stash}{div_pages_loop_if_prev_first} = ($page_numbers[0] == 2);
    local $ctx->{__stash}{div_pages_loop_if_next_last} = ($page_numbers[$#page_numbers] == $pages_count - 1);
    my $vars = $ctx->{__stash}{vars} ||= {};
    foreach $page_no (@page_numbers) {
        local $ctx->{__stash}{loop_header} = ($ctr == 0) ? 1 : 0;
        local $ctx->{__stash}{loop_footer} = ($ctr == $page_numbers_count - 1) ? 1 : 0;
        local $vars->{__first__} = !$ctr;
        local $vars->{__last__} = ($ctr == $page_numbers_count - 1);
        local $vars->{__odd__} = ($ctr % 2) == 0;
        local $vars->{__even__} = ($ctr % 2) == 1;
        local $vars->{__counter__} = $ctr + 1;
#        $ctx->stash('loop_header', ($ctr == 0) ? 1 : 0);
#        $ctx->stash('loop_footer', ($ctr == scalar(@page_numbers) - 1) ? 1 : 0);
        if ((($ctr == 0 && $page_numbers[$ctr] != 1) ||
             ($ctr != 0 && $page_numbers[$ctr - 1] != $page_numbers[$ctr] - 1))
            && $delimiter && !$count) {
            local $ctx->{__stash}{page_no} = -1;
#            $ctx->stash('page_no', -1);
            defined(my $out = $builder->build($ctx, $tokens))
                or return $ctx->error($ctx->errstr);
            push @res, $out;
        }
        local $ctx->{__stash}{page_no} = $page_no;
#        $ctx->stash('page_no', $page_no);
        defined(my $out = $builder->build($ctx, $tokens))
            or return $ctx->error($ctx->errstr);
        push @res, $out;
        if ($ctr == scalar(@page_numbers) - 1 &&
            $page_numbers[$ctr] != $ctx->stash('pages_count') &&
            $delimiter && !$count) {
            local $ctx->{__stash}{page_no} = -1;
            $ctx->stash('page_no', -1);
            defined(my $out = $builder->build($ctx, $tokens))
                or return $ctx->error($ctx->errstr);
            push @res, $out;
        }
        $ctr++;
    }
    my $glue = $args->{glue} || '';
    $res = join $glue, @res;
    $res;
}

# MTDivPagesEntryCount tag
sub div_pages_entry_count {
    my ($ctx, $args) = @_;

    $ctx->stash('entry_count');
}

sub div_pages_content_count {
    my ($ctx, $args) = @_;

    $ctx->stash('content_count');
}

# MTDivPagesNumber tag
# MTDivPagesNumberLink tag
sub div_pages_number {
    my ($ctx, $args) = @_;

    my $tag = lc $ctx->stash('tag');
    my $number = $ctx->stash('page_no');
    if ($number != -1) {
        if ($tag eq 'divpagesnumber') {
            return $number;
        }
        else {
            my $amp = $args->{ajax} ? '&' : '&amp;';
            return $amp . 'page=' . $number;
        }
    }
    else {
        return $ctx->stash('number_delimiter');
    }
}

# MTDivPagesLink tag
sub div_pages_link {
    my ($ctx, $args) = @_;

    my $page_no = $ctx->stash('page_no');
    if ($page_no != -1) {
        return &div_pages_cur_page_link($ctx, $args) . "?page=$page_no";
    }
    else {
        return '';
    }
}

# MTDivPagesFirstPagelink tag
sub div_pages_first_page_link {
    my ($ctx, $args) = @_;

    &div_pages_cur_page_link($ctx, $args) . "?page=first";
}

# MTDivPagesLastPagelink tag
sub div_pages_last_page_link {
    my ($ctx, $args) = @_;

    &div_pages_cur_page_link($ctx, $args) . "?page=last";
}

# MTDivPagesPrevPagelink tag
sub div_pages_prev_page_link {
    my ($ctx, $args) = @_;

    my $cur_page_no = $ctx->stash('cur_page_no');
    &div_pages_cur_page_link($ctx, $args) . "?page=" . ($cur_page_no - 1);
}

# MTDivPagesNextPagelink tag
sub div_pages_next_page_link {
    my ($ctx, $args) = @_;

    my $cur_page_no = $ctx->stash('cur_page_no');
    &div_pages_cur_page_link($ctx, $args) . "?page=" . ($cur_page_no + 1);
}

# MTDivPagesPrevPageNumber tag
sub div_pages_prev_page_number {
    my ($ctx, $args) = @_;

    my $cur_page_no = $ctx->stash('cur_page_no');
    ($cur_page_no - 1);
}

# MTDivPagesNextPageNumber tag
sub div_pages_next_page_number {
    my ($ctx, $args) = @_;

    my $cur_page_no = $ctx->stash('cur_page_no');
    ($cur_page_no + 1);
}

# MTDivPagesLastPageNumber tag
sub div_pages_last_page_number {
    my ($ctx, $args) = @_;

    my $pages_count = $ctx->stash('pages_count');
    $pages_count;
}

# MTDivPagesIfFirstPage tag
sub div_pages_if_first_page {
    my ($ctx, $args) = @_;

    my $cur_page_no = $ctx->stash('cur_page_no');
    return ($cur_page_no == 1) ? 1 : 0;
}

# MTDivPagesIfLastPage tag
sub div_pages_if_last_page {
    my ($ctx, $args) = @_;

    my $cur_page_no = $ctx->stash('cur_page_no');
    my $pages_count = $ctx->stash('pages_count');
    return ($cur_page_no == $pages_count) ? 1 : 0;
}

# MTDivPagesIfPrevPage tag
sub div_pages_if_prev_page {
    my ($ctx, $args) = @_;

    my $cur_page_no = $ctx->stash('cur_page_no');
    return ($cur_page_no > 1) ? 1 : 0;
}

# MTDivPagesIfNextPage tag
sub div_pages_if_next_page {
    my ($ctx, $args) = @_;

    my $cur_page_no = $ctx->stash('cur_page_no');
    my $pages_count = $ctx->stash('pages_count');

    return ($cur_page_no < $pages_count) ? 1 : 0;
}

# MTDivPagesIfDelimiter tag
sub div_pages_if_delimiter {
    my ($ctx, $args) = @_;

    my $page_no = $ctx->stash('page_no');
    return ($page_no == -1) ? 1 : 0;
}

# MTDivPagesIfCurPage tag
sub div_pages_if_cur_page {
    my ($ctx, $args) = @_;

    my $cur_page_no = $ctx->stash('cur_page_no');
    my $page_no = $ctx->stash('page_no');
    return ($page_no == $cur_page_no) ? 1 : 0;
}

# MTDivPagesCount tag
sub div_pages_count {
    my ($ctx, $args) = @_;

    $ctx->stash('pages_count');
}

# MTDivPagesIfMulti tag
sub div_pages_if_multi {
    my ($ctx, $args) = @_;

    my $pages_count = $ctx->stash('pages_count');
    return ($pages_count >= 2) ? 1 : 0;
}

# MTDivPagesCurPageLink tag
sub div_pages_cur_page_link {
    my ($ctx, $args) = @_;

    my $mode = $args->{'mode'} || 'absolute';
    my $uri = $ctx->stash('cur_page_uri');
    if ($mode eq 'relative') {
        return $uri;
    }
    else {
        return 'http://' . $ctx->invoke_handler('bloghost', $args) . $uri;
    }
}

# MTDivPagesCurPageNumber tag
sub div_pages_cur_page_number {
    my ($ctx, $args) = @_;

    $ctx->stash('cur_page_no');
}

# MTDivPagesLoopHeader tag
sub div_pages_loop_header {
    my ($ctx, $args) = @_;

    $ctx->stash('loop_header');
}

# MTDivPagesLoopFooter tag
sub div_pages_loop_footer {
    my ($ctx, $args) = @_;

    $ctx->stash('loop_footer');
}

# MTDivPagesLoopIfPrevPage tag
sub div_pages_loop_if_prev_page {
    my ($ctx, $args) = @_;

    $ctx->stash('div_pages_loop_if_prev_page');
}

# MTDivPagesLoopIfNextPage tag
sub div_pages_loop_if_next_page {
    my ($ctx, $args) = @_;

    $ctx->stash('div_pages_loop_if_next_page');
}

# MTDivPagesLoopIfPrevFirst tag
sub div_pages_loop_if_prev_first {
    my ($ctx, $args) = @_;

    $ctx->stash('div_pages_loop_if_prev_first');
}

# MTDivPagesLoopIfNextLast tag
sub div_pages_loop_if_next_last {
    my ($ctx, $args) = @_;

    $ctx->stash('div_pages_loop_if_next_last');
}

# MTDivPagesCurPageFirstEntryNumber tag
sub div_pages_cur_page_first_entry_number {
    my ($ctx, $args) = @_;

    my $cur_page = $ctx->stash('cur_page_no');
    my $per_page = $ctx->stash('per_page');
    ($cur_page - 1) * $per_page + 1;
}

# MTDivPagesCurPageLastEntryNumber tag
sub div_pages_cur_page_last_entry_number {
    my ($ctx, $args) = @_;

    my $cur_page = $ctx->stash('cur_page_no');
    my $per_page = $ctx->stash('per_page');
    if (lc $ctx->stash('tag') eq 'divpagescurpagelastentrynumber') {
        if ($cur_page * $per_page > $ctx->stash('entry_count')) {
            return $ctx->stash('entry_count');
        }
        else {
            return $cur_page * $per_page;
        }
    }
    elsif (lc $ctx->stash('tag') eq 'divpagescurpagelastcontentnumber') {
        my $app = MT->instance;
        my $total_count = $ctx->stash('fj_srhcnt_found_count') || ($app->param('total_count') + 0) || $ctx->stash('content_count');
        if ($cur_page * $per_page > $total_count) {
            return $total_count;
        }
        else {
            return $cur_page * $per_page;
        }
    }
}

# MTDivPagesBuildParams tag
sub div_pages_build_params {
    my ($ctx, $args) = @_;
    my $app = MT->instance;
    if ($args->{post}) {
        $args->{per_page} = $ctx->var('dp_per_page');
        $args->{total_count} = $ctx->var('dp_total_count');
    }
    return _build_params($app, $args);
}

sub div_pages_cgi_link {
    my ($ctx, $args) = @_;

    require MT::Request;

    my $req = MT::Request->instance;
    my $tag = lc $ctx->stash('tag') eq 'divpagescgilink'
            ? 'realtimerebuildlink' : 'realtimerebuildphplink';
    my $key = 'FJ_divpages:' . $tag . ':' . $args->{tmpl_name} . ':' . $args->{tmpl_type};
    my $link;
    if ($req->stash($key)) {
        $link = $req->stash($key);
    }
    else {
        $args->{per_page} = $ctx->var('dp_per_page') || 10;
        $args->{total_count} = $ctx->var('dp_total_count');
        local $ctx->{__stash}{tag} = $tag;
        $link = $ctx->invoke_handler($tag, $args);
        $req->stash($key, $link);
    }
    if ($args->{build_params}) {
        my $app = MT->instance;
        $args->{php} = 1 if ($tag eq 'realtimerebuildphplink');
        my $params = _build_params($app, $args);
        my $amp = $args->{ajax} ? '&' : '&amp;';
        $link .= $amp . $params if ($params);
    }
    return $link;
}

sub _build_params {
    my ($app, $args) = @_;

    my @fields;
    if ($args->{name}) {
        my $fields_str = $args->{name};
        @fields = split /\s*,\s*/, $fields_str;
        @fields = grep { $_ ne 'build_params' && $_ ne 'name' } @fields;
    }
    else {
        my $q = $app->param;
        my @p = $q->param;
        my %exclude;
        if ($args->{exclude}) {
            for my $field (split /\s*,\s*/, $args->{exclude}) {
                $exclude{$field} = 1;
            }
            $exclude{build_params} = 1;
            $exclude{name} = 1;
            @fields = grep { !$exclude{$_} } @p;
        }
    }

    if ($args->{post}) {
        my $html = '';
        @fields = grep {
                      $_ ne 'blog_id' && $_ ne 'tmpl_id' &&
                      $_ ne 'per_page' && $_ ne 'total_count'
                  } @fields;
        my @p_fields = ( 'blog_id', 'tmpl_id', @fields );
        for my $field (@p_fields) {
            my @p = $app->param($field);
            my $f = (scalar(@p) > 1 && $args->{php}) ? $field . '[]' : $field;
            for my $p (@p) {
                $html .= '<input type="hidden" name="' . $f . '" value="' . encode_html($p) . "\" />\n";
            }
        }
        $html .= '<input type="hidden" name="per_page" value="' . $args->{per_page} . "\" />\n";
        $html .= '<input type="hidden" name="total_count" value="' . $args->{total_count} . "\" />\n";
        return $html;
    }
    else {
        my $amp = $args->{ajax} ? '&' : '&amp;';
        my @param = ();
        for my $field (@fields) {
            my @p = $app->param($field);
            my $f = (scalar(@p) > 1 && $args->{php}) ? $field . '[]' : $field;
            for my $p (@p) {
                push @param, $f . '=' . encode_url($p);
            }
        }
        return join $amp, @param;
    }
}

1;
